
# q-expansion of newform 432.8.a.y, downloaded from the LMFDB on 15 March 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 432
weight = 8
poly_data = [-63, -702, -257, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[-219, 194, 8, -1], 2], [[2895, 1826, 20, -7], 2], [[-3768, -812, 4, 4], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [26, 1, 0, 0], [123, -1, -1, 0], [526, -1, -4, -1], [589, 15, -1, 2], [-1034, 19, 12, -5], [-1379, -84, -4, 6], [4462, 43, -40, -2], [37680, 168, -144, 20], [19564, -172, -156, -34], [-10581, -835, -35, 28], [70176, 888, 296, -46], [12800, -278, -38, 50], [-207402, 1871, 312, 78], [440780, 2390, 472, 158], [-470498, -849, -1852, -239], [217279, 6495, 527, 330], [62425, -1256, 888, -12], [-2336212, 246, 1480, 386], [-115415, 10, 3434, -204], [234029, -18491, -843, 34], [-3956596, 518, -472, -846], [5705934, -14753, 828, 311], [2399165, -5116, -3916, -1158], [8014896, 23088, 8696, 334], [5741447, 7177, -17815, 212], [-12056022, -12971, -8244, 187], [3297106, -12794, 2870, 794], [8170302, -68577, 2660, 1785], [9408476, -39858, -6194, 2320], [-13178440, -33972, 10304, -600], [18095202, 16457, -18508, -467], [9493395, -42106, 26070, 2598], [2852548, 62242, 4984, -1890], [21060193, 17233, -9503, -5170], [23251822, 83638, 24550, -10118], [27948291, -74064, 17824, -8302], [29658678, -70257, -64568, 6650], [-6570336, 253368, 7976, 8946], [-25619004, 7482, -1200, 22372], [50213529, -102613, -19141, 8062], [40708698, -27487, 26112, -14944], [62259263, -135466, 2534, -2802], [22058034, 124853, 14584, 19366], [93968543, 196481, -76927, 17984], [95598503, 148618, -42118, -23534], [63923780, -434664, 2152, -36362], [149658492, -207226, -136880, 9444], [79068274, 45492, 145812, 33572], [25445912, -433596, -9272, 12450], [271887548, 221382, 6768, 1292], [96342873, 551154, 103394, -34606], [-96508600, -434132, 22264, -35690], [-320523116, 321842, 4224, 47480], [157029220, -799574, 944, 33660], [138456934, 208143, 240912, -45476], [166047961, 540013, 261837, 436], [7058170, -138330, 61654, 41018], [-146390240, -88176, -57344, 46104], [190117192, 552934, -185962, 28826], [-361530978, 968491, 126240, -18272], [165390456, -376930, 216526, 14986], [297353658, 1212385, 484496, -24324], [-46811949, -1277582, 88418, 5202], [37256436, -1026206, -348608, -1960], [-116605877, 914394, -316182, 73390], [221243637, -774688, -572048, 42398], [316574316, -1122986, -915064, 99122], [-392286597, -1951813, -999557, -23352], [295251988, 798250, -81864, -219090], [-1148459190, -289583, -304936, 21198], [-537393985, -828171, -176475, 106250], [-473688973, 353937, 909777, -43464], [136674385, 1040758, -110186, 2332], [106402896, -669024, -189288, -143226], [212534378, -1717247, 1210872, 148158], [-296755330, -2035058, 60286, 240802], [99279516, -1293362, -1509240, -85102], [172861929, -583280, -1478368, -148226], [-162258570, -1014085, 1966452, -14419], [-1786412323, 737927, 199079, -79780], [-1034194354, 5099955, -281424, 75868], [251333194, -6246330, 435958, -365350], [-1658448396, -1594556, -512652, 133082], [237995708, 3708150, -1623968, 231040], [936765194, 2578645, -87204, 180167], [-1651547890, -2012978, 1579582, -10398], [53959526, 3054015, -1124224, -464912], [-749622869, -1470257, -162849, -154114], [-164766138, -1007389, -1315756, -200627], [397723220, 939226, 617736, -43302], [670412843, 2435669, -2672795, -664642], [787761450, -6001755, -24164, -220961], [-497095152, 4256068, -801692, 287036], [891258222, 5398179, 1690384, -356532], [-1169802222, 3766229, 1986728, 730490], [-1496127158, -4267155, -2285484, -314307], [562303169, -6972252, -1115980, 302766], [-383255523, -9844155, -3040603, 313876], [-3065992493, 6591626, -845654, 287900], [-3193382602, 1080831, -986120, 24302], [1076929072, 3127440, -2848232, -533442], [-1700411050, -8856061, -116852, 332283], [334786773, -4552990, -2215662, -1120854], [3664424051, 10998902, 5501366, 376632], [1130710114, -8754447, -248484, -1120353], [-4326116620, -8271142, 978936, -1610], [-978361008, -9932344, 323776, -85304], [-442943150, -3089578, -3361018, 248826], [756131323, 15676069, 2242597, 310960], [-102560377, 7074331, 4937403, -1046516], [2082605370, -2654003, -3554148, -1481441], [-2912471155, 16355548, -68484, 466508], [3251206361, -6781355, 6941221, 665254], [83534036, -23769646, 2063680, -185232], [2060072160, -9685754, -590698, 1153662], [4019717360, -2513168, 1163032, -95730], [5161004940, 9153086, -3894496, 495872], [-1811041620, 13772382, 1948400, 1721236], [-1063372057, -5753337, -3712793, 1785316], [1234945435, -9258392, 425656, -115258], [3193418354, 4435005, -3943008, 281888], [1206517096, -26184500, 7538696, -167246], [-2271131600, -16972658, -6105698, 329326], [-2651108394, 12710511, 3335256, 1260758], [-7342297843, 5263053, -3804675, -269222], [3269850964, 30943250, -85376, 1503032], [-4254802068, -3951672, 9968824, -964030], [-347922190, -16711198, -8870414, 765470], [8328410824, 2286186, -1792774, -639554], [-9429860162, -15111973, -10520960, -456344], [5647250493, 35162399, 2209279, -360764], [1174970789, -46891347, 502109, -76150], [9426907606, 11199739, 6579460, -583943], [13313490959, -6393680, -1119760, -1365096], [-1617287244, -4293822, -5471040, -1294456], [4180402517, -3444352, 16305936, 1610174], [7398937304, -28702348, 1545112, 1858070], [1131692236, -1027474, -15375376, -719892], [8844396080, -24833798, -2378838, 2126906], [3356501634, -28533291, -5130864, 2961404], [3955260193, -14834513, -3447985, 679404], [4726193702, -20752189, 1616772, -1009687], [-173925119, 24503607, 2963479, 4197604], [-4358446092, 1186570, 9717160, 4295378], [-13062704005, 45466291, 2377795, 341518], [-466940148, -6755186, -1414672, -2699460], [5267875263, 9853378, 12295906, -2565060], [84861286, 16887015, 22272616, 552634], [-9212726829, -24722351, -97631, -4164010], [-8869728278, 39386469, 22120188, -2115769], [2433876355, -6964298, 6514646, -2491460], [479517908, -22335726, 18229696, 1616520], [-6068497639, 19724956, 22542412, 4010074], [9456005464, 7622292, -28716488, 874118], [10334737964, -7921024, -31439824, 2435114], [-6479174084, 46780654, -9006696, -1879354], [14491739715, 57296538, 18198442, 2411054], [-7693404642, -19700229, -26626448, -3395220], [13651124946, 22275657, 7845276, -1897969], [-1440698202, -21591981, -9769516, 1489837], [-8190946043, 56503643, 6451531, 1952930], [3747445962, 67278325, -2870692, 7687711], [-23238922860, -54809542, -23335224, -930502], [12908848026, 62481369, 22454744, -1242218], [-4501666225, -43837219, 28745581, -2856082], [-23400696698, 80062710, 7062438, -3433878], [5563991453, -44843150, 6272274, -889524], [-24939164242, -10127669, 23068184, -2733274], [1910613084, -98423138, 6967992, -4825674], [-25117660770, 99204730, 12323786, -3055338], [21944740598, -49359041, 8505304, -2738586], [10141788925, 99562620, 8697660, -5731704], [-18154414265, -88797615, -367679, 249266], [-10435138868, 81378830, -2194544, 4727380], [-10601830760, -105025798, 19448650, -8299162], [12929412726, -19102297, 34359552, 5942376], [-14172183259, -18385903, 21324369, -6896264], [-26000683066, 69067636, 14460884, 2954804], [-6391748411, -16297671, 5156169, -2287362], [2290874996, -93809910, 16993256, 9437842], [-32390286438, -47695894, -10433222, 5256022], [7162994994, -51242479, 28262068, 3763237], [13965783096, 8482436, -31339080, 8903854], [3658227780, -13073454, -391256, -4535150], [-43920785855, 3554569, 28151897, -2173738], [-2373041241, -68766718, -27888302, 9977574], [-6441706883, -86137946, 20257686, -5901774], [-36314030058, -159745113, -24173072, -7578956], [-8881483055, -66252038, -27334870, -8764438], [-16897362228, -107516770, 3091936, -1948192], [-44264739481, 120876184, 14244568, 2494552], [27511348214, -85702617, -41153472, 12777832], [-36967176798, 47396433, -37263844, -2912577], [-9190833326, -78469175, -2888484, -451137], [4027155955, 44404190, -36643714, -2389364], [4200114985, 71743083, -13805685, -2024804], [-2366339930, -54503061, 34913004, 2684555], [-65347761204, -83854562, 7717792, 7377144], [15133896406, 50177023, -26255192, -11385966], [26370650905, -67405407, -52612127, -4600584], [-23527369663, 86340293, 21181669, -5924772], [34595320981, 27625472, -3637584, -6405382], [-41414925240, 126566220, -28745896, -667114], [16375585404, -83973178, 84619280, 10472180], [-2196567383, -61472397, 20415299, 12398918], [-5695714026, -59755165, -38268404, 821707], [39378992218, -56576123, 35775492, 5002705], [14748506753, -61910986, 60023414, 13603544], [-14759117625, 125282556, -6387684, 7978876], [-7456204806, -61400335, -15689792, 11833952], [24075887429, 68287319, 58660679, -12056478], [39340394726, 80257979, -102714084, -8358985], [-20186108392, 150103572, -38530920, 9716230], [39955722719, 139929772, 92256572, 6077598], [-4547198804, 158023988, -5264444, -1901298], [-13011477250, -20810377, -62466276, -3168889], [49597857828, 64043426, 4441224, 16124594], [15197337012, 205163370, 87115496, -11678118], [4334094113, -214676993, 78262831, 12070490], [-26342315676, -42947552, -22954832, -7172114], [-19387501320, -295263004, 44183864, -18819474], [46034343652, 197105932, 23896124, 684698], [16324849674, -327000859, -48550164, -508677], [93286501799, 103684717, -6140451, 6681810], [-15799040994, 329329047, -4701076, 9369163], [18793270422, -367658361, -46633840, 15181724], [54104776383, -166537721, 19987415, -15203258], [-17960004530, -14101537, 77330492, 3114511], [49033947909, -4926561, 10345071, 3240454], [-89182105192, 44798048, 102205888, -14522892], [2871106060, -42205238, -4314838, 26821428], [17282683538, 48702081, 42234228, 3128253], [102928373247, -458244456, 47053992, 2129326], [94038198914, -196997059, -69299528, -3507082], [-14989976491, -313259862, 23388970, -22745296], [-50346043882, -242749617, -49026728, -3519810], [-22620599428, -61246602, -14111744, 20312680], [82948352476, 178703334, -81509120, -8710312], [43016131956, -389853910, -11234232, 15573042], [-50588528195, -48143738, 33802550, -8644166], [-36992537695, -65370667, -16154907, -20545730], [32413626715, -23215893, 13278667, 20430436], [25412683302, -192840013, 19268788, 28241949], [-54855938278, 120710473, 27161352, 14473282], [71951244965, -27565437, 98247107, 23070392], [-2645017008, -182415160, 89007680, 23710728], [100068079616, 140983080, -3839000, -14482008], [135300096344, 159762124, 21811632, -39305644], [88741909698, -264374050, -115822514, 5048274], [-49856505486, 537449697, -107013468, -8613831], [2350463582, 128414707, 157698312, -15285470], [-9968660099, 119248698, 149743050, -718818], [54069669018, -213637575, 49282424, -12007418], [16680220674, -485560015, -146862004, 13409779], [23001961169, 81561291, 228706875, 30621906], [113088373016, 260689614, 143616894, 12962970], [-214294172736, 322460000, 34919680, -13082464], [-39817402814, -55167790, -277009310, -33812386], [68585854259, 175201193, 102572265, -11097960], [-13574698470, 408039421, -63892836, -24750977], [51022886917, -318248199, 104512873, -1750894], [44097670959, -251651337, 26332391, -7573982], [68422459892, 119711890, 187187376, 14544140], [12600921880, 40934298, -111231126, -48146090], [50965449502, -173098293, -8581248, 23949448], [165792239692, 59655254, -46417768, -24716274], [154471655368, 36382380, -128369556, 2204872], [79886708134, -188894433, 81287104, -28274352], [61778870959, -11596581, 14351691, 41867922], [-191005926185, -478470910, -187940494, -11584126], [-77702644154, -582664264, -248348424, 1807000], [-108536410843, 145196991, 123836319, -44901596], [-135283295978, 191754470, 70691094, 24410282], [14775048343, -621140231, 115632313, 22531816], [18813701094, 222565547, -50113204, -41085829], [44437344494, 34243086, 197910014, 28657154], [-163703425838, 70327082, 28813050, 23451702], [-55839735416, 23534700, -60675048, -31473970], [-140690653514, -145135073, -110353512, 33763310], [-191413439428, 324729642, -78779830, 12842364], [68929824544, -355427360, 78414928, -8534212], [23527824011, -195724527, -73197743, -6508424], [147707718775, 322678094, -17305138, 16559184], [-118327786910, 3408373, -180062400, -38310320], [-129491744503, 258186586, 2294154, 28488074], [-52141725036, 31383490, 54361808, -23446812], [-273839523797, 151771261, 230478893, 10758186], [228140378554, 167517197, 16322524, -9702617], [-97307417590, -156703711, 287849416, -33898678], [224989767576, -997632044, 95749080, 678414], [265854133124, -89338454, -38732640, -25092288], [-21483586752, -294006488, -189859192, 56670970], [-28647590106, 481533260, 259564652, 64047420], [-30541376080, 673528544, 158649384, 88808450], [-35161143917, -55536533, -322572517, -51733590], [51573362586, 225898961, -132843696, -28158660], [-145476481032, 461032004, -325583464, -42656810], [-283329202715, -257800482, 189764766, -5341368], [-197354521045, -142128316, 240011172, -34585036], [6590747536, -549271984, -293616472, 55643914], [270543132028, 957230292, 213734820, 9632422], [123940376416, 84337576, 285793016, 28065566], [172384151367, 637142122, 260606314, -47198752], [-124594784218, -353152274, -317005090, 16710322], [299916773466, -54043971, 145075580, -6534913], [74765868893, 374732829, -207495715, 53138600], [-78268990528, 28019616, 442241152, 12550776], [197808703043, 529805145, 108068857, 47760524], [226391841558, 1434504439, 263061824, 13988968], [8744787720, 992580708, -127496752, -373884], [-176396744264, 91087260, 316209744, -16617052], [159466923083, 768588586, -230744406, 7516112], [137974803874, 1224574517, -64887392, 101346208], [-145995252946, -193630654, 314956370, -43722738], [-215873831226, 776530331, -53950388, -21032397], [457884253830, -219871137, -140997872, -77006452], [-231974333626, -69114710, -168358150, -17078458], [139733830362, -994772931, -413619524, 14896335], [-264612931087, 313939886, -118015650, -3044454], [-41441047949, 416731092, -264603596, 90046580], [222953096812, 365566902, 101493736, -89761558], [574204740740, -752210762, 131418518, 27532996], [114969373124, 727335418, 103725648, 89611580], [18564150885, 1098578930, 379590706, -46918472], [-446897070432, -807033094, 188473706, -17186846], [102599256936, 331275712, -139435936, 103462348], [90808611130, 676338081, -11110384, 31431308], [341763420294, 1205667127, 532955176, -1600350], [-1554988523, -1279135807, -56955519, -120829336], [-403163099064, -1167691908, 48767048, -82616502], [-107665613055, 1388849841, -105388303, -74195368], [191704701720, 1044012076, -162239248, -2207044], [48352707216, -1436750552, 418887976, -49972864], [-117977177364, 662850782, 372813904, -91651044], [304562369371, 313944969, -75549799, 20970590], [-376342700838, 92507117, -409586964, -66777061], [-19010226226, -1105375226, 167231670, -44894758], [811041746796, -306046918, -6711142, 28068500], [-11460258581, -908241851, -273702139, -1915416], [-404557402350, 137032369, 143474340, 164234193], [27421026372, -1006016782, 223909320, -82595878], [-407411009307, -1164646721, -68138737, -46224458], [94517187800, -1316095068, -696611384, 63583026], [-166831400342, 2452411493, -166047428, 121793319], [300332409138, -307078308, 191931772, -63817860], [151767636152, 749181426, 229915778, -115859010], [-64967041236, 652395806, 484656176, -46329780], [53836877290, 1748915193, 227359856, 135565676], [393649899285, -398775882, 184106870, -78290072], [349387104449, 2151856322, 564171586, 58914640], [-243069227728, 482336328, 46970768, 112774028], [129635266596, -723066928, 621919328, 91397582], [43782404963, 524448519, 302834263, -156138], [91016414338, 391223649, 684964284, 27961199], [-513458384300, 1231292698, -92494712, 35649202], [-533593878136, -20131052, -87959936, 26976632], [-290160889672, 59914084, 179304856, -12108642], [-771547362876, 503621914, 326511600, -9036108], [48882300407, 2165768587, 70842027, -129122532], [-107973473256, 2144497716, -245549192, -57524730], [579081548748, -288237858, -377270304, 61857528], [-618926725594, -1623794717, -311214620, -73987687], [-228875211241, 299402792, -480027960, -4294516], [51839897559, 2424314586, 473749018, -125171752], [246879126246, 1289195919, 749638160, 15237140], [548327598515, -744335971, 385639437, 5533170], [523083057295, -981214606, 154362226, -2400704], [-538358977510, 2235817293, 123890284, 70864019], [48053950575, -1703283126, -695399158, -188595832], [-230480761552, 1689224368, 475827512, 145771670], [215913246248, -910329844, 707585064, -6556958], [-586406212765, 1039860893, -235110739, -39672322], [69757815413, 864879505, 338476401, 133372132], [-537311345010, 411130567, 696521860, -93266359], [-610240547232, 846218160, 550617856, -103605000], [522580241887, -2778571910, -187349846, 113708362], [-112636393932, 2851182242, -278582880, 93148360], [-97020721675, 441220542, 1109950958, -112335926], [-93822900582, -1814667255, -552914552, 100391378], [477902154640, 2363860952, 83528368, 121554820], [473491970025, -112062053, -385368965, -21073052], [-854204788452, -1579907074, -580477640, 8707382], [-128773482715, -172278938, -58311642, 149044104], [707045328588, -1143016578, -165063616, 78385448], [344443580580, -513513038, -574752718, 2956656], [12394330313, 1534789233, -587891247, 39816372], [209851105739, 1762041222, -221605994, 128504750], [385884863610, 736098761, -424162536, -103597514], [61683450718, 1816087160, -141412168, -71743112], [689289004160, 415693296, -247573712, 57436460], [-225020693346, -643198753, -653456524, 171652149], [-369136551963, -478795538, 1415601166, 78087996], [331072925838, 141560299, -767133496, -53619446], [434800110481, -1718049084, -852725292, -17815186], [699852007785, -1684333841, -401473377, -191672470], [489980732226, -1019068151, -684307964, 88685209], [1045384182880, -330589366, -134949062, -114097550], [116988650888, -746458324, -643519656, -225004210], [-616409862466, 5463762, 638714338, 143020638], [175881073560, -354367692, 819334520, -50436178], [-343579081516, -3621044968, -543698008, -2486130], [-181234561280, 13574744, 622655512, 103949334], [-164104935100, 2827373650, 280984616, -2738398], [139856002076, -1138155592, 1040355816, 59813562], [-476951337047, 299569785, 163525193, -167657210], [535391778684, -491782882, -721471512, 91044002], [303277347691, -1021129528, -709446984, -225082318], [313992643554, 1498570649, 245085252, -323268511], [477802577258, 2293206310, 418024406, -254031174], [-617298993052, -55367678, 256008072, -125848982], [274374897782, 2454543659, 234499236, 267799681], [-516970184461, 1330176792, 626231560, -167681358], [429196602729, 1398521070, 188424814, -308131344], [-573846351778, 331354971, 433276272, 332363108], [658979406870, -909325265, 597671352, 122278950], [-847834243727, 764165671, -47910361, 424512480], [-1134065508240, 1653226616, -296580832, 152811472], [-95529218692, 1735321118, 743295400, 167170362], [1113105900226, 876401077, -944912944, 105359468], [-872579591146, 257080506, -491391478, 30531222], [1317649686542, 3137683291, 138428984, 144286558], [-1653190525000, -1355779604, 11703008, 159837968], [429179997298, 1948218654, -259870066, 134887842], [-978793223840, -600104336, -44583680, 244435072], [-1382586325610, -176529341, 330173612, 38130419], [285893761980, 1689206614, -394288256, 121245920], [-763282887984, 2100162162, 765646434, 122588338], [-51462007400, 382566276, 196118920, -58229110]]
